﻿Imports System.Collections.ObjectModel
Imports PageLayoutControl.PrinterLayerDrawingUtilities
Imports ThinkGeo.MapSuite.Drawing
Imports ThinkGeo.MapSuite.Shapes
Imports ThinkGeo.MapSuite.Styles

<Serializable()>
Public Class CustomPrinterLayerShapeStyle
  Inherits AreaStyle

  Public Property PrinterLayerDrawingType As EnumPrinterLayerDrawingType = EnumPrinterLayerDrawingType.Rectangle
  Public Property OutlineWidth As Single = 1
  Public Property OutlineColor As GeoColor = GeoColors.Black
  Public Property DrawingPreview As Boolean = False
  Public Property ArrowOrientation As EnumPageLayoutArrowDirection
  Public Property ArrowSize As Single = 2
  Public Property ArrowLineWidth As Single = 1
  Public Property ArrowColor As GeoColor = GeoColors.Black
  Public Property DrawingShape As BaseShape
  Public Property LineDrawingStyle As LineStyle
  Public Property AreaDrawingStyle As AreaStyle


  Public Sub New(eDrawingType As EnumPrinterLayerDrawingType)
    MyBase.New
    PrinterLayerDrawingType = eDrawingType
  End Sub
  Public Sub New(oDrawingShape As BaseShape, eDrawingType As EnumPrinterLayerDrawingType)
    MyBase.New
    PrinterLayerDrawingType = eDrawingType
    DrawingShape = oDrawingShape
    LineDrawingStyle = LineStyles.CreateSimpleLineStyle(GeoColors.Black, 1, True)
    AreaDrawingStyle = AreaStyles.CreateSimpleAreaStyle(GeoColors.Transparent, GeoColors.Black, 1)
  End Sub

  Protected Overrides Sub DrawCore(oFeatures As IEnumerable(Of Feature),
                                   oGeoCanvas As GeoCanvas,
                                   oLabelsInThisLayer As Collection(Of SimpleCandidate),
                                   oLabelsInAllLayers As Collection(Of SimpleCandidate))

    MyBase.DrawCore(oFeatures, oGeoCanvas, oLabelsInThisLayer, oLabelsInAllLayers)
    Dim dCurrentScale As Double = oGeoCanvas.CurrentScale
    Dim sngActualSizeScale As Single = 12 * oGeoCanvas.Dpi
    Dim sngScaledLineWidth As Single = CSng(OutlineWidth * (sngActualSizeScale / dCurrentScale))

    If DrawingPreview Then
      sngScaledLineWidth = OutlineWidth
    End If

    For Each oFeature As Feature In oFeatures
      '***** Get the extent of the rectangle in screen coordinates
      Dim oFeatureExtent = oFeature.GetBoundingBox
      If oFeatureExtent.Width < 1 Or oFeatureExtent.Height < 1 Then
        Return
      End If

      Dim oScreenExtent = ExtentHelper.ToScreenCoordinate(oGeoCanvas.CurrentWorldExtent, oFeatureExtent, oGeoCanvas.Width, oGeoCanvas.Height)
      Select Case PrinterLayerDrawingType
        Case EnumPrinterLayerDrawingType.Ellipse, EnumPrinterLayerDrawingType.Circle
          Dim oCenterPoint As ScreenPointF = New ScreenPointF(oScreenExtent.CenterX, oScreenExtent.CenterY)
          Dim sngHeight As Single = oScreenExtent.Height
          Dim sngWidth As Single = oScreenExtent.Width

          '***** Set the starting and ending point of the arrow so that it has the orientation
          '***** chosen by the user
          oGeoCanvas.DrawEllipse(oCenterPoint, sngWidth, sngHeight, New GeoPen(OutlineColor, sngScaledLineWidth), New GeoSolidBrush(GeoColors.Transparent), DrawingLevel.LabelLevel, 0, 0, PenBrushDrawingOrder.BrushFirst)
        Case EnumPrinterLayerDrawingType.Rectangle, EnumPrinterLayerDrawingType.Square
          Dim oRectanglePoints As New List(Of ScreenPointF())
          Dim oScreenPoints(4) As ScreenPointF
          With oScreenExtent
            oScreenPoints(0) = New ScreenPointF(.MinimumX, .MaximumY)
            oScreenPoints(1) = New ScreenPointF(.MaximumX, .MaximumY)
            oScreenPoints(2) = New ScreenPointF(.MaximumX, .MinimumY)
            oScreenPoints(3) = New ScreenPointF(.MinimumX, .MinimumY)
            oScreenPoints(4) = New ScreenPointF(.MinimumX, .MaximumY)
            oRectanglePoints.Add(oScreenPoints)
          End With
          oGeoCanvas.DrawArea(oRectanglePoints, New GeoPen(OutlineColor, sngScaledLineWidth), New GeoSolidBrush(GeoColors.Transparent), DrawingLevel.LabelLevel, 0, 0, PenBrushDrawingOrder.BrushFirst)
        Case EnumPrinterLayerDrawingType.Triangle
          Dim oTrianglePoints As New List(Of ScreenPointF())
          Dim oScreenPoints(3) As ScreenPointF
          With oScreenExtent
            oScreenPoints(0) = New ScreenPointF(.MinimumX, .MaximumY) '<--- Lower Left
            oScreenPoints(1) = New ScreenPointF(.MaximumX, .MaximumY) '<--- Lower Right
            oScreenPoints(2) = New ScreenPointF(CSng(.MinimumX + .Width / 2), .MinimumY) '<--- Center top
            oScreenPoints(3) = New ScreenPointF(.MinimumX, .MaximumY) '<--- Lower Left
            oTrianglePoints.Add(oScreenPoints)
          End With
          oGeoCanvas.DrawArea(oTrianglePoints, New GeoPen(OutlineColor, sngScaledLineWidth), New GeoSolidBrush(GeoColors.Transparent), DrawingLevel.LabelLevel, 0, 0, PenBrushDrawingOrder.BrushFirst)
        Case EnumPrinterLayerDrawingType.Diamond
          Dim oDiamondPoints As New List(Of ScreenPointF())
          Dim oScreenPoints(4) As ScreenPointF
          With oScreenExtent
            oScreenPoints(0) = New ScreenPointF(CSng(.MinimumX + .Width / 2), .MaximumY) '<--- Bottom center
            oScreenPoints(1) = New ScreenPointF(.MaximumX, .MaximumY - CSng(.Height / 2)) '<--- Right side
            oScreenPoints(2) = New ScreenPointF(CSng(.MinimumX + .Width / 2), .MinimumY) '<--- Center top
            oScreenPoints(3) = New ScreenPointF(.MinimumX, .MaximumY - CSng(.Height / 2)) '<--- Left side
            oScreenPoints(4) = New ScreenPointF(CSng(.MinimumX + .Width / 2), .MaximumY) '<--- Bottom center
            oDiamondPoints.Add(oScreenPoints)
          End With
          oGeoCanvas.DrawArea(oDiamondPoints, New GeoPen(OutlineColor, sngScaledLineWidth), New GeoSolidBrush(GeoColors.Transparent), DrawingLevel.LabelLevel, 0, 0, PenBrushDrawingOrder.BrushFirst)
        Case EnumPrinterLayerDrawingType.Line, EnumPrinterLayerDrawingType.Polygon
          Dim oShapeToDraw As New Collection(Of BaseShape) From {DrawingShape}
          Dim oCurrentExtent As RectangleShape = oFeatures(0).GetShape.GetBoundingBox
          Dim oShapeExtent As RectangleShape = DrawingShape.GetBoundingBox
          Dim dX As Double = oCurrentExtent.UpperRightPoint.X - oShapeExtent.UpperRightPoint.X
          Dim dY As Double = oCurrentExtent.UpperRightPoint.Y - oShapeExtent.UpperRightPoint.Y
          DrawingShape.TranslateByOffset(dX, dY)
          DrawingShape.ScaleTo(oCurrentExtent.Width / oShapeExtent.Width)
          If PrinterLayerDrawingType = EnumPrinterLayerDrawingType.Line Then
            LineDrawingStyle.Draw(oShapeToDraw, oGeoCanvas, oLabelsInAllLayers, oLabelsInAllLayers)
          Else
            AreaDrawingStyle.Draw(oShapeToDraw, oGeoCanvas, oLabelsInAllLayers, oLabelsInAllLayers)
          End If
        Case EnumPrinterLayerDrawingType.Arrow
          DrawArrow(oFeatures, oGeoCanvas, oLabelsInThisLayer, oLabelsInAllLayers, ArrowSize, ArrowLineWidth, ArrowOrientation, ArrowColor, DrawingPreview)
      End Select

    Next oFeature
  End Sub
End Class
