﻿Imports System.Collections.ObjectModel
Imports ThinkGeo.MapSuite.Drawing
Imports ThinkGeo.MapSuite.Layers
Imports ThinkGeo.MapSuite.Shapes
Imports ThinkGeo.MapSuite.Styles

Public Class PrinterLayerDrawingUtilities
  Public Enum EnumPageLayoutArrowDirection
    Left = 1
    Right = 2
    Up = 3
    Down = 4
    UpperRight = 5
    UpperLeft = 6
    LowerRight = 7
    LowerLeft = 8
  End Enum
  Public Enum EnumPrinterLayerDrawingType
    Ellipse = 1
    Circle = 2
    Rectangle = 4
    Square = 3
    Triangle = 5
    Diamond = 6
    Line = 7
    Polygon = 8
    Arrow = 9
    None = 10
  End Enum

  Public Shared Sub DrawArrow(oFeatures As IEnumerable(Of Feature),
                              oGeoCanvas As GeoCanvas,
                              oLabelsInThisLayer As Collection(Of SimpleCandidate),
                              oLabelsInAllLayers As Collection(Of SimpleCandidate),
                              ArrowSize As Single, LineWidth As Single, Orientation As EnumPageLayoutArrowDirection,
                              ArrowColor As GeoColor, ByRef DrawingPreview As Boolean)

    For Each oFeature As Feature In oFeatures
      '***** Calcuate a scalefactor for the size of the arrow head, based on the size currently selected.
      '***** This will cause the size of the arrowhead to scale properly when the page layout is zoomed.
      Dim dCurrentScale As Double = oGeoCanvas.CurrentScale
      Dim sngActualSizeScale As Single = 12 * oGeoCanvas.Dpi
      Dim sngScaledArrowSize As Single = CSng(ArrowSize * (sngActualSizeScale / dCurrentScale))
      Dim sngScaledLineWidth As Single = CSng(LineWidth * (sngActualSizeScale / dCurrentScale))
      If DrawingPreview Then
        sngScaledArrowSize = ArrowSize
        sngScaledLineWidth = LineWidth
      End If

      '***** Get the extent of the rectangle in screen coordinates
      Dim oScreenExtent = ExtentHelper.ToScreenCoordinate(oGeoCanvas.CurrentWorldExtent, oFeature.GetBoundingBox, oGeoCanvas.Width, oGeoCanvas.Height)
      'Debug.Print("Width = " + oScreenExtent.Width.ToString + ", Height = " + oScreenExtent.Height.ToString)
      '***** Get the starting and ending point for the total arrow, including shaft
      Dim oStartPoint As ScreenPointF
      Dim oEndPoint As ScreenPointF

      '***** Set the starting and ending point of the arrow so that it has the orientation
      '***** chosen by the user
      Select Case Orientation
        Case EnumPageLayoutArrowDirection.Left
          oStartPoint = New ScreenPointF(oScreenExtent.MaximumX, oScreenExtent.MaximumY - CInt(oScreenExtent.Height / 2))
          oEndPoint = New ScreenPointF(oScreenExtent.MinimumX, oScreenExtent.MaximumY - CInt(oScreenExtent.Height / 2))
        Case EnumPageLayoutArrowDirection.Right
          oStartPoint = New ScreenPointF(oScreenExtent.MinimumX, oScreenExtent.MaximumY - CInt(oScreenExtent.Height / 2))
          oEndPoint = New ScreenPointF(oScreenExtent.MaximumX, oScreenExtent.MaximumY - CInt(oScreenExtent.Height / 2))
        Case EnumPageLayoutArrowDirection.Up
          oStartPoint = New ScreenPointF(oScreenExtent.MinimumX + CInt(oScreenExtent.Width / 2), oScreenExtent.MaximumY)
          oEndPoint = New ScreenPointF(oScreenExtent.MinimumX + CInt(oScreenExtent.Width / 2), oScreenExtent.MinimumY)
        Case EnumPageLayoutArrowDirection.Down
          oStartPoint = New ScreenPointF(oScreenExtent.MinimumX + CInt(oScreenExtent.Width / 2), oScreenExtent.MinimumY)
          oEndPoint = New ScreenPointF(oScreenExtent.MinimumX + CInt(oScreenExtent.Width / 2), oScreenExtent.MaximumY)
        Case EnumPageLayoutArrowDirection.UpperRight
          oStartPoint = New ScreenPointF(oScreenExtent.MinimumX, oScreenExtent.MaximumY)
          oEndPoint = New ScreenPointF(oScreenExtent.MaximumX, oScreenExtent.MinimumY)
        Case EnumPageLayoutArrowDirection.LowerLeft
          oStartPoint = New ScreenPointF(oScreenExtent.MaximumX, oScreenExtent.MinimumY)
          oEndPoint = New ScreenPointF(oScreenExtent.MinimumX, oScreenExtent.MaximumY)
        Case EnumPageLayoutArrowDirection.UpperLeft
          oStartPoint = New ScreenPointF(oScreenExtent.MaximumX, oScreenExtent.MaximumY)
          oEndPoint = New ScreenPointF(oScreenExtent.MinimumX, oScreenExtent.MinimumY)
        Case EnumPageLayoutArrowDirection.LowerRight
          oStartPoint = New ScreenPointF(oScreenExtent.MinimumX, oScreenExtent.MinimumY)
          oEndPoint = New ScreenPointF(oScreenExtent.MaximumX, oScreenExtent.MaximumY)
      End Select

      '***** Draw the arrow
      mDrawArrow(oGeoCanvas, oStartPoint, oEndPoint, sngScaledArrowSize, sngScaledLineWidth, ArrowColor)

    Next oFeature
    '***** Set this flag back in case we were drawing a preview
    DrawingPreview = False

  End Sub
  Private Shared Sub mDrawArrow(oGeoCanvas As GeoCanvas, oArrowStart As ScreenPointF, oArrowEnd As ScreenPointF, sngArrowSize As Single, sngLineWidth As Single, ArrowColor As GeoColor)
    Dim oGeoPen As GeoPen = New GeoPen(ArrowColor, sngLineWidth)
    Dim oScreenPoints As New Collection(Of ScreenPointF)

    '***** oArrowPoint, the point at the tip of the arrow
    Dim oArrowPoint As ScreenPointF = oArrowEnd
    '***** dArrowLenght is used for calculating the size of the arrow and also the
    '***** angle of the arrow tip. In order to get a more pleasing shape and to maintain
    '***** scaling a certain amount (10 * sngArrowSize) is subtracted from this length,
    '***** which is calculating using the square root of the squares of the sums of the two sides.
    Dim dArrowLength As Double = Math.Sqrt(
                                 Math.Pow(Math.Abs(oArrowStart.X - oArrowEnd.X), 2) +
                                 Math.Pow(Math.Abs(oArrowStart.Y - oArrowEnd.Y), 2)) -
                                 (10 * sngArrowSize)

    '***** I have not thoroughly analyzed all of these calculations, having found this sample on the web
    '***** https://www.codeproject.com/Questions/125049/Draw-an-arrow-with-big-cap
    '***** but I have modified it enough to  draw a nice arrow
    '***** SET 04/01/2019
    Dim dArrowAngle As Double = Math.Atan2(Math.Abs(oArrowStart.Y - oArrowEnd.Y), Math.Abs(oArrowStart.X - oArrowEnd.X))
    Dim dPointX, dPointY As Double
    Dim iArrowFactor As Integer = 3

    '***** Create a point that lies on the arrow line, the distance of the length of the arrow head only
    If oArrowStart.X > oArrowEnd.X Then
      dPointX = oArrowStart.X - (Math.Cos(dArrowAngle) * (dArrowLength - (iArrowFactor * sngArrowSize)))
    Else
      dPointX = Math.Cos(dArrowAngle) * (dArrowLength - (iArrowFactor * sngArrowSize)) + oArrowStart.X
    End If

    If oArrowStart.Y > oArrowEnd.Y Then
      dPointY = oArrowStart.Y - (Math.Sin(dArrowAngle) * (dArrowLength - (iArrowFactor * sngArrowSize)))
    Else
      dPointY = (Math.Sin(dArrowAngle) * (dArrowLength - (iArrowFactor * sngArrowSize))) + oArrowStart.Y
    End If

    '***** oArrowPointBack, The middle point that intersects the line on the backside of the arrow
    Dim oArrowPointBack As ScreenPointF = New ScreenPointF(CSng(dPointX), CSng(dPointY))

    Dim dAngleB As Double = Math.Atan2((iArrowFactor * sngArrowSize), (dArrowLength - (iArrowFactor * sngArrowSize)))
    Dim dAngleC As Double = Math.PI * (90 - (dArrowAngle * (180 / Math.PI)) -
                                      (dAngleB * (180 / Math.PI))) / 180
    Dim dSecondaryLength As Double = (iArrowFactor * sngArrowSize) / Math.Sin(dAngleB)

    '***** Create a point off the edge of the line that makes up the left side of the barb of the arrow
    If oArrowStart.X > oArrowEnd.X Then
      dPointX = oArrowStart.X - (Math.Sin(dAngleC) * dSecondaryLength)
    Else
      dPointX = (Math.Sin(dAngleC) * dSecondaryLength) + oArrowStart.X
    End If

    If oArrowStart.Y > oArrowEnd.Y Then
      dPointY = oArrowStart.Y - (Math.Cos(dAngleC) * dSecondaryLength)
    Else
      dPointY = (Math.Cos(dAngleC) * dSecondaryLength) + oArrowStart.Y
    End If

    '***** oArrowPointLeft, the point on the left side of the arrow, as viewed from the front of the arrow
    Dim oArrowPointLeft As ScreenPointF = New ScreenPointF(CSng(dPointX), CSng(dPointY))
    dAngleC = dArrowAngle - dAngleB

    If oArrowStart.X > oArrowEnd.X Then
      dPointX = oArrowStart.X - (Math.Cos(dAngleC) * dSecondaryLength)
    Else
      dPointX = (Math.Cos(dAngleC) * dSecondaryLength) + oArrowStart.X
    End If

    If oArrowStart.Y > oArrowEnd.Y Then
      dPointY = oArrowStart.Y - (Math.Sin(dAngleC) * dSecondaryLength)
    Else
      dPointY = (Math.Sin(dAngleC) * dSecondaryLength) + oArrowStart.Y
    End If

    '***** oArrowPointLeft, the point on the right side of the arrow, as viewed from the front of the arrow
    Dim oArrowPointRight As ScreenPointF = New ScreenPointF(CSng(dPointX), CSng(dPointY))

    '***** Create a polygon to draw the tip of the arrow (the pointy part)
    Dim oPolygon As New PolygonShape()

    Dim oWorldPt As PointShape

    '***** Convert to world coordinates for drawing the arrow tip
    oWorldPt = ExtentHelper.ToWorldCoordinate(oGeoCanvas.CurrentWorldExtent, oArrowPoint.X, oArrowPoint.Y, oGeoCanvas.Width, oGeoCanvas.Height)
    oPolygon.OuterRing.Vertices.Add(New Vertex(oWorldPt.X, oWorldPt.Y))

    oWorldPt = ExtentHelper.ToWorldCoordinate(oGeoCanvas.CurrentWorldExtent, oArrowPointLeft.X, oArrowPointLeft.Y, oGeoCanvas.Width, oGeoCanvas.Height)
    oPolygon.OuterRing.Vertices.Add(New Vertex(oWorldPt.X, oWorldPt.Y))

    oWorldPt = ExtentHelper.ToWorldCoordinate(oGeoCanvas.CurrentWorldExtent, oArrowPointBack.X, oArrowPointBack.Y, oGeoCanvas.Width, oGeoCanvas.Height)
    oPolygon.OuterRing.Vertices.Add(New Vertex(oWorldPt.X, oWorldPt.Y))

    oWorldPt = ExtentHelper.ToWorldCoordinate(oGeoCanvas.CurrentWorldExtent, oArrowPointRight.X, oArrowPointRight.Y, oGeoCanvas.Width, oGeoCanvas.Height)
    oPolygon.OuterRing.Vertices.Add(New Vertex(oWorldPt.X, oWorldPt.Y))

    oWorldPt = ExtentHelper.ToWorldCoordinate(oGeoCanvas.CurrentWorldExtent, oArrowPoint.X, oArrowPoint.Y, oGeoCanvas.Width, oGeoCanvas.Height)
    oPolygon.OuterRing.Vertices.Add(New Vertex(oWorldPt.X, oWorldPt.Y))

    '***** Brush to fill the drawing of the arrow
    Dim oBrush As GeoBrush = New GeoSolidBrush(ArrowColor)

    '***** Draw the shaft of the arrow
    oScreenPoints.Add(oArrowStart)
    oScreenPoints.Add(oArrowPointBack)
    oGeoCanvas.DrawLine(oScreenPoints, oGeoPen, DrawingLevel.LabelLevel, 0, 0)
    '***** Draw the tip of the arrow
    oGeoCanvas.DrawArea(oPolygon, oBrush, DrawingLevel.LabelLevel)

  End Sub
  Public Shared Function GetClickPoint(oWorldExtent As RectangleShape) As PointShape
    '***** Get upper left point in inches.
    Dim dXInches As Double = PrinterHelper.ConvertLength(oWorldExtent.UpperLeftPoint.X, PrintingUnit.Point, PrintingUnit.Inch)
    Dim dYInches As Double = PrinterHelper.ConvertLength(oWorldExtent.UpperLeftPoint.Y, PrintingUnit.Point, PrintingUnit.Inch)
    Return New PointShape(dXInches, dYInches)

  End Function
  Public Shared Function GetClickPoint(oPoint1 As PointShape, oPoint2 As PointShape) As PointShape
    Dim dXInches As Double = PrinterHelper.ConvertLength(oPoint1.X, PrintingUnit.Point, PrintingUnit.Inch)
    Dim dYInches As Double = PrinterHelper.ConvertLength(oPoint1.Y, PrintingUnit.Point, PrintingUnit.Inch)
    Dim oPoint1Inches = New PointShape(dXInches, dYInches)

    dXInches = PrinterHelper.ConvertLength(oPoint2.X, PrintingUnit.Point, PrintingUnit.Inch)
    dYInches = PrinterHelper.ConvertLength(oPoint2.Y, PrintingUnit.Point, PrintingUnit.Inch)
    Dim oPoint2Inches = New PointShape(dXInches, dYInches)

    Dim oPageExtent As RectangleShape = CreateExtentFromEndPoints(oPoint1Inches, oPoint2Inches)
    Return oPageExtent.UpperLeftPoint

  End Function
  Public Shared Function CreateExtentFromEndPoints(oPoint1 As PointShape, oPoint2 As PointShape) As RectangleShape
    Dim dMinX As Double
    Dim dMaxX As Double
    Dim dMinY As Double
    Dim dMaxY As Double

    If oPoint1.X < oPoint2.X Then
      dMinX = oPoint1.X
      dMaxX = oPoint2.X
    Else
      dMinX = oPoint2.X
      dMaxX = oPoint1.X
    End If

    If oPoint1.Y < oPoint2.Y Then
      dMinY = oPoint1.Y
      dMaxY = oPoint2.Y
    Else
      dMinY = oPoint2.Y
      dMaxY = oPoint1.Y
    End If

    Dim oExtent As New RectangleShape(dMinX, dMaxY, dMaxX, dMinY)
    Return oExtent

  End Function

End Class
