﻿Imports System.Collections.ObjectModel
Imports System.Drawing.Printing
Imports ThinkGeo.MapSuite
Imports ThinkGeo.MapSuite.Layers
Imports ThinkGeo.MapSuite.WinForms
Imports ThinkGeo.MapSuite.Drawing
Imports ThinkGeo.MapSuite.Styles
Imports ThinkGeo.MapSuite.Shapes
Imports PageLayoutControl.PrinterLayerDrawingUtilities
Public Class SilvicsPageLayout
  Inherits WinformsMap
  'Public Shared Property ThePageLayoutControl As SilvicsPageLayout

  Private mcsPRINTER_OVERLAY_NAME As String = "PrinterOverlay"
  Private mcsPAGE_LAYER_NAME As String = "PageLayer"
  Private mcsTITLE As String = "LabelLayer"
  Private mcsMAP_PRINTER_LAYER As String = "MapPrinterLayer"
  Private WithEvents moPrinterOverlay As CustomPrinterInteractiveOverlay
  Public Property InMemDisplayLayer As InMemoryFeatureLayer
  Public Sub New()
    MyBase.New()
    Me.DrawingQuality = DrawingQuality.HighSpeed
    Me.CompositingQuality = Drawing2D.CompositingQuality.HighSpeed
    CreateDefaultLayout()
    Dim oCustomTrackOverlay As New CustomPrinterTrackOverlay()
    oCustomTrackOverlay.TrackMode = TrackMode.None
    TrackOverlay = oCustomTrackOverlay
    oCustomTrackOverlay.PageLayoutParent = Me
    Dim oStyle As LineStyle = oCustomTrackOverlay.TrackShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle

    oStyle.CenterPen.EndCap = DrawingLineCap.ArrowAnchor
    oStyle.InnerPen.EndCap = DrawingLineCap.ArrowAnchor
    oStyle.OuterPen.EndCap = DrawingLineCap.ArrowAnchor
    Cursor = Cursors.Cross
  End Sub
  Public Sub StopDrawing(sToolKey As String)

    With CType(TrackOverlay, CustomPrinterTrackOverlay)
      .TrackOverlayCancelled()
      .TrackShapeLayer.InternalFeatures.Clear()
    End With

    If sToolKey = My.Resources.SelectObject Then
      With CType(TrackOverlay, CustomPrinterTrackOverlay)
        .TrackMode = TrackMode.None
        .DrawingType = EnumPrinterLayerDrawingType.None
      End With

      mSetSelectabilityOfObjects(SelectabilityMode.Selectable)
    End If
    Refresh()
  End Sub
  Public Sub StartDrawing(eDrawingType As EnumPrinterLayerDrawingType)
    With CType(TrackOverlay, CustomPrinterTrackOverlay)
      .DrawingType = eDrawingType
      mSetSelectabilityOfObjects(SelectabilityMode.NonSelectable)

      Select Case eDrawingType
        Case EnumPrinterLayerDrawingType.Arrow, EnumPrinterLayerDrawingType.Line
          .TrackMode = TrackMode.Line
        Case EnumPrinterLayerDrawingType.Arrow, EnumPrinterLayerDrawingType.Polygon
          .TrackMode = TrackMode.Polygon
        Case EnumPrinterLayerDrawingType.Ellipse
          .TrackMode = TrackMode.Ellipse
        Case EnumPrinterLayerDrawingType.Diamond, EnumPrinterLayerDrawingType.Triangle, EnumPrinterLayerDrawingType.Rectangle
          .TrackMode = TrackMode.Rectangle
        Case EnumPrinterLayerDrawingType.Square
          .TrackMode = TrackMode.Square
        Case EnumPrinterLayerDrawingType.Circle
          .TrackMode = TrackMode.Circle
      End Select
    End With

  End Sub

  Private Sub mSetSelectabilityOfObjects(eSelectabilityMode As SelectabilityMode)
    Dim oPrinterOverlay As CustomPrinterInteractiveOverlay = CType(InteractiveOverlays(mcsPRINTER_OVERLAY_NAME), CustomPrinterInteractiveOverlay)
    For Each oPrinterLayer As PrinterLayer In oPrinterOverlay.PrinterLayers
      oPrinterLayer.SelectabilityMode = eSelectabilityMode
    Next oPrinterLayer
  End Sub

  Public Sub CreateDefaultLayout()
    Dim oPagePrinterLayer As PagePrinterLayer
    Dim oExtentRectangle As RectangleShape

    '***** Setup the map unit, you want to always use feet or meters for printer layout maps
    MapUnit = GeographyUnit.Feet

    '***** Create the default zoom levels for the layout map
    ZoomLevelSet = New PrinterZoomLevelSet(MapUnit, PrinterHelper.GetPointsPerGeographyUnit(MapUnit))

    '***** Set the background color to gray so there is contrast with the white page
    BackgroundOverlay.BackgroundBrush = New GeoSolidBrush(GeoColor.StandardColors.LightGray)

    '***** Create the PrinterInteractiveOverlay to contain all of the PrinterLayers.
    '***** The interactive overlay allows the layers to be interacted with
    moPrinterOverlay = New CustomPrinterInteractiveOverlay()
    InteractiveOverlays.Add(mcsPRINTER_OVERLAY_NAME, moPrinterOverlay)
    InteractiveOverlays.MoveToBottom(mcsPRINTER_OVERLAY_NAME)


    '***** Create the PagePrinterLayer which shows the page boundary and 
    '***** is the area the user will arrange all of the other layers on top of.
    oPagePrinterLayer = New PagePrinterLayer(PrinterPageSize.AnsiA, PrinterOrientation.Portrait)
    With oPagePrinterLayer
      .Open()
      .BackgroundMask = AreaStyles.CreateSimpleAreaStyle(GeoColor.StandardColors.White, GeoColor.SimpleColors.Black, 1)
      moPrinterOverlay.PrinterLayers.Add(mcsPAGE_LAYER_NAME, oPagePrinterLayer)

      '***** Get the page's extent, scale it up by 10%
      oExtentRectangle = CType(RectangleShape.ScaleUp(.GetPosition(), 10), RectangleShape)
      '***** Set oExtentRectangle as the default current extent
      CurrentExtent = oExtentRectangle
    End With

    '***** Set the minimum sscale of the map to the last zoom level
    MinimumScale = ZoomLevelSet.ZoomLevel20.Scale
    ''***** Add a container for the map layers
    mAddMapLayer()
    'AddScaleLineLayer()

    ''***** Add a default title
    'mAddCenteredTitleLabel()
    'AddLogoImage()
    Refresh()
  End Sub
  Private Sub mAddCenteredTitleLabel()
    Dim oLabelPrinterLayer As LabelPrinterLayer = New LabelPrinterLayer()
    Dim oPageBoundingbox As RectangleShape
    Dim oPrinterInteractiveOverLay As PrinterInteractiveOverlay

    '***** Setup the text and the font..
    oLabelPrinterLayer.Text = "Page Layout"
    oLabelPrinterLayer.Font = New GeoFont("Arial", 10, DrawingFontStyles.Bold)
    'oLabelPrinterLayer.FontBrush = New GeoSolidBrush(GeoColor.StandardColors.Black)

    '***** Set the labels position so that is it centered on the page one inch from the top                        
    oPageBoundingbox = GetPageBoundingBox(PrintingUnit.Inch)
    Dim oLabelCenter As PointShape = New PointShape()
    oLabelCenter.X = oPageBoundingbox.UpperRightPoint.X - oPageBoundingbox.Width / 2
    oLabelCenter.Y = oPageBoundingbox.UpperLeftPoint.Y - 0.75

    oLabelPrinterLayer.PrinterWrapMode = PrinterWrapMode.AutoSizeText
    oLabelPrinterLayer.SetPosition(5, 0.5, oLabelCenter, PrintingUnit.Inch)

    '***** Find the PrinterInteractiveOverlay so we can add the new LabelPrinterLayer
    oPrinterInteractiveOverLay = CType(InteractiveOverlays(mcsPRINTER_OVERLAY_NAME), PrinterInteractiveOverlay)
    oPrinterInteractiveOverLay.PrinterLayers.Add(mcsTITLE, oLabelPrinterLayer)


  End Sub
  Private Sub mAddMapLayer()
    Dim oMapPrinterLayer As New MapPrinterLayer()
    Dim oPageBoundingbox As RectangleShape = GetPageBoundingBox(PrintingUnit.Inch)
    Dim oPrinterOverLay As CustomPrinterInteractiveOverlay

    '***** Create the MapPrinterLayer and set the position
    oMapPrinterLayer.BackgroundMask = AreaStyles.CreateSimpleAreaStyle(GeoColor.StandardColors.White, GeoColor.SimpleColors.Black, 1)

    '***** Set the maps position slightly below the pages center and 8 inches wide and 7 inches tall
    oPageBoundingbox = GetPageBoundingBox(PrintingUnit.Inch)
    oMapPrinterLayer.SetPosition(7.5, 10, oPageBoundingbox.GetCenterPoint().X, oPageBoundingbox.GetCenterPoint().Y, PrintingUnit.Inch)

    '***** Add the MapPrinterLayer to the PrinterInteractiveOverlay
    oMapPrinterLayer.MapUnit = GeographyUnit.Feet

    oPrinterOverLay = CType(InteractiveOverlays(mcsPRINTER_OVERLAY_NAME), CustomPrinterInteractiveOverlay)
    oPrinterOverLay.PrinterLayers.Add(mcsMAP_PRINTER_LAYER, oMapPrinterLayer)
  End Sub
  Public Sub Print()

    '***** Get the printer for printing
    Dim oPrinterSettings As PrinterSettings = DotNetUtils.GetPrinter()
    If oPrinterSettings Is Nothing Then
      Return
    End If

    Dim oPrinterOverLay As PrinterInteractiveOverlay = PrinterOverlay
    Dim oPagePrinterLayer As PagePrinterLayer = PrinterPage

    Dim oPrintDocument As New PrintDocument()
    With oPrintDocument
      .PrinterSettings = oPrinterSettings
      .DefaultPageSettings.Landscape = True
      If oPagePrinterLayer.Orientation = PrinterOrientation.Portrait Then
        .DefaultPageSettings.Landscape = False
      End If
      .DefaultPageSettings.PaperSize = GetPrintPreviewPaperSize(oPagePrinterLayer)
    End With

    Dim oPrinterCanvas As PrinterGeoCanvas = New PrinterGeoCanvas()

    '***** Create a rectangle to represent the drawing area for the document
    oPrinterCanvas.DrawingArea = New Rectangle(0, 0, Convert.ToInt32(oPrintDocument.DefaultPageSettings.PrintableArea.Width), Convert.ToInt32(oPrintDocument.DefaultPageSettings.PrintableArea.Height))
    '***** Start the drawing process
    oPrinterCanvas.BeginDrawing(oPrintDocument, oPagePrinterLayer.GetBoundingBox(), MapUnit)

    '***** Loop through all of the PrintingLayers in the PrinterInteractiveOverlay 
    '***** and print all of the except for the PagePrinterLayer                
    Dim labelsInAllLayers As Collection(Of SimpleCandidate) = New Collection(Of SimpleCandidate)()
    For Each oPrinterLayer As PrinterLayer In oPrinterOverLay.PrinterLayers
      oPrinterLayer.IsDrawing = True
      If Not (TypeOf oPrinterLayer Is PagePrinterLayer) Then
        oPrinterLayer.Draw(oPrinterCanvas, labelsInAllLayers)
      End If
      oPrinterLayer.IsDrawing = False
    Next oPrinterLayer

    oPrinterCanvas.EndDrawing()
  End Sub
  Public Sub AddScaleLineLayer()
    '*****  Get the PrinterInteractiveOverlay
    Dim oPrinterOverLay As PrinterInteractiveOverlay = PrinterOverlay

    '***** Grab the MapPrinterLayer as we need to pass that into the
    '*****  scaleLinePrinterLayer so it can be synced up to the map itself in real-time
    Dim oPageLayoutMap As MapPrinterLayer = PageLayoutMap

    '***** Create the scaleLinePrinterLayer and pass in the MapPrinterLayer
    Dim oScaleLineLayer As New ScaleLinePrinterLayer(oPageLayoutMap)

    oScaleLineLayer.MapUnit = GeographyUnit.Meter

    '***** Set the scale line position offset from the page center and 1.25 inches wide and .25 inches tall
    Dim oPageBoundingbox As RectangleShape = GetPageBoundingBox(PrintingUnit.Inch)
    oScaleLineLayer.SetPosition(2.5, 0.25, oPageBoundingbox.GetCenterPoint().X - 2.25, oPageBoundingbox.GetCenterPoint().Y - 4.5, PrintingUnit.Inch)

    '***** Add the ScaleLinePrinterLayer to the PrinterInteractiveOverlay
    oPrinterOverLay.PrinterLayers.Add("ScaleLineLayer", oScaleLineLayer)
  End Sub
  Public Sub AddLogoImage()
    '**** Load the image to a GeoImage 
    Dim oBitmap As New Bitmap(My.Resources.SmallSilvicsLogo)
    Dim oSilvicsLogo As GeoImage = DotNetUtils.GeoImageFromBitmap(oBitmap)

    '***** Create the ImagePrinterLayer using the GeoImage
    Dim oImagePrinterLayer As ImagePrinterLayer = New ImagePrinterLayer(oSilvicsLogo)

    ' Set the imagePrinterLayer position offset from the page center and .75 inches wide and .75 inches tall
    Dim oPageBoundingbox As RectangleShape = GetPageBoundingBox(PrintingUnit.Inch)
    oImagePrinterLayer.SetPosition(1.75, 0.75, oPageBoundingbox.GetCenterPoint().X + 2.75, oPageBoundingbox.GetCenterPoint().Y + 3.8, PrintingUnit.Inch)

    ' Add the imagePrinterLayer to the PrinterInteractiveOverlay
    Dim oPrinterOverLay As PrinterInteractiveOverlay = PrinterOverlay
    oPrinterOverLay.PrinterLayers.Add("ImageLayer", oImagePrinterLayer)
  End Sub
  Public Sub PrintPreview()
    Dim oPrinterOverLay As PrinterInteractiveOverlay = PrinterOverlay
    Dim oPagePrinterLayer As PagePrinterLayer = PrinterPage

    Dim oPrintDialog As PrintDialog = New PrintDialog()
    Dim oPrintDocument As PrintDocument = New PrintDocument()
    oPrintDocument.DefaultPageSettings.Landscape = True
    If oPagePrinterLayer.Orientation = PrinterOrientation.Portrait Then
      oPrintDocument.DefaultPageSettings.Landscape = False
    End If

    oPrintDocument.DefaultPageSettings.PaperSize = GetPrintPreviewPaperSize(oPagePrinterLayer)

    Dim oPrinterCanvas As PrinterGeoCanvas = New PrinterGeoCanvas()
    oPrinterCanvas.DrawingArea = oPrintDocument.DefaultPageSettings.Bounds
    oPrinterCanvas.BeginDrawing(oPrintDocument, oPagePrinterLayer.GetBoundingBox(), GeographyUnit.Feet)

    ' Loop through all of the PrintingLayer in the PrinterInteractiveOverlay and print all of the
    ' except for the PagePrinterLayer                
    Dim oLabelsInAllLayers As Collection(Of SimpleCandidate) = New Collection(Of SimpleCandidate)()
    For Each oPrinterLayer As PrinterLayer In oPrinterOverLay.PrinterLayers
      oPrinterLayer.IsDrawing = True
      If Not (TypeOf oPrinterLayer Is PagePrinterLayer) Then
        oPrinterLayer.Draw(oPrinterCanvas, oLabelsInAllLayers)
      End If
      oPrinterLayer.IsDrawing = False
    Next oPrinterLayer

    oPrinterCanvas.Flush()

    Dim oPrintPreviewDialog As PrintPreviewDialog = New PrintPreviewDialog()
    oPrintPreviewDialog.WindowState = FormWindowState.Maximized
    oPrintPreviewDialog.Document = oPrintDocument
    Dim eDialogResult As DialogResult = oPrintPreviewDialog.ShowDialog()
    If eDialogResult = Windows.Forms.DialogResult.OK Then
      Debug.WriteLine("snot")
    End If
  End Sub

  Public Function GetPrintPreviewPaperSize(ByVal pagePrinterLayer As PagePrinterLayer) As PaperSize
    Dim printPreviewPaperSize As PaperSize = New PaperSize("AnsiA", 850, 1100)
    Select Case pagePrinterLayer.PageSize
      Case PrinterPageSize.AnsiA
        printPreviewPaperSize = New PaperSize("AnsiA", 850, 1100)
      Case PrinterPageSize.AnsiB
        printPreviewPaperSize = New PaperSize("AnsiB", 1100, 1700)
      Case PrinterPageSize.AnsiC
        printPreviewPaperSize = New PaperSize("AnsiC", 1700, 2200)
      Case PrinterPageSize.AnsiD
        printPreviewPaperSize = New PaperSize("AnsiD", 2200, 3400)
      Case PrinterPageSize.AnsiE
        printPreviewPaperSize = New PaperSize("AnsiE", 3400, 4400)
      Case PrinterPageSize.Custom
        printPreviewPaperSize = New PaperSize("Custom Size", CInt(pagePrinterLayer.CustomWidth), CInt(pagePrinterLayer.CustomHeight))
      Case Else
    End Select

    Return printPreviewPaperSize
  End Function
  Private Function GetPageBoundingBox(ByVal unit As PrintingUnit) As RectangleShape
    '***** This helper method gets the pages bounding box in the unit requested
    Dim oPrinterOverLay As PrinterInteractiveOverlay = CType(InteractiveOverlays(mcsPRINTER_OVERLAY_NAME), PrinterInteractiveOverlay)
    Dim oPageLayer As PagePrinterLayer = CType(oPrinterOverLay.PrinterLayers(mcsPAGE_LAYER_NAME), PagePrinterLayer)
    Return oPageLayer.GetPosition(unit)

  End Function
  Public Sub SetGeoExtent(ByVal oExtent As RectangleShape)
    PageLayoutMap.MapExtent = oExtent
  End Sub
  Public Sub ViewEntirePageLayout()
    Dim oPrinterPage As PagePrinterLayer = PrinterPage
    If Not oPrinterPage Is Nothing Then
      Dim oCurrentExtent As RectangleShape = oPrinterPage.GetBoundingBox

      oCurrentExtent.ScaleUp(10)
      'PageLayoutMap.SetPosition(oCurrentExtent)
      CurrentExtent = oCurrentExtent

      Refresh()
    End If
  End Sub
  Public ReadOnly Property PrinterOverlay As CustomPrinterInteractiveOverlay
    Get
      Return moPrinterOverlay
    End Get
  End Property
  Public ReadOnly Property PrinterPage As PagePrinterLayer
    Get
      Dim oPrinterPage As PagePrinterLayer = Nothing
      Try
        If Not PrinterOverlay Is Nothing Then
          oPrinterPage = CType(PrinterOverlay.PrinterLayers(mcsPAGE_LAYER_NAME), PagePrinterLayer)
        End If
      Catch
      End Try
      Return oPrinterPage
    End Get
  End Property

  Public ReadOnly Property PageLayoutMap As MapPrinterLayer
    Get
      Dim oPageLayoutMap As MapPrinterLayer = Nothing
      If Not PrinterOverlay Is Nothing Then
        Try
          oPageLayoutMap = TryCast(PrinterOverlay.PrinterLayers(mcsMAP_PRINTER_LAYER), MapPrinterLayer)
        Catch
        End Try
      End If
      Return oPageLayoutMap
    End Get
  End Property
  Protected Overrides Sub OnKeyDown(e As KeyEventArgs)
    If e.KeyCode = Keys.Escape AndAlso CType(TrackOverlay, CustomPrinterTrackOverlay).DrawingType <> EnumPrinterLayerDrawingType.None Then
      StopDrawing(String.Empty)
    End If
    MyBase.OnKeyDown(e)
  End Sub
  'Protected Overrides Sub OnMouseDown(e As MouseEventArgs)
  '  Dim oScreenExtent As RectangleShape = PageLayoutMap.GetPosition(PrintingUnit.Point)
  '  Dim dX As Single
  '  Dim dY As Single
  '  dX = e.X - 360
  '  dY = e.Y
  '  Dim o As PointShape = ExtentHelper.ToWorldCoordinate(PageLayoutMap.MapExtent, dX, dY, CSng(oScreenExtent.Width), CSng(oScreenExtent.Height))
  '  Debug.WriteLine("X = " + o.X.ToString + ", Y = " + o.Y.ToString)

  '  Dim oScreenPointF As New ScreenPointF(e.X, e.Y)

  '  Dim oLocationPoint As New PointShape(e.X, e.Y)

  '  Dim p = PrinterHelper.ConvertMapPrinterCoordinateToScreenCoordinate(oLocationPoint, PageLayoutMap, CurrentExtent, Width, Height)

  '  MyBase.OnMouseDown(e)
  'End Sub
  'Protected Overrides Sub OnMapClick(e As MapClickWinformsMapEventArgs)
  '  Dim oScreenExtent As RectangleShape = PageLayoutMap.GetPosition(PrintingUnit.Point)
  '  Dim dX As Single
  '  Dim dY As Single
  '  dX = e.ScreenX
  '  dY = e.ScreenY
  '  'Dim o As PointShape = ExtentHelper.ToWorldCoordinate(PageLayoutMap.MapExtent, dX, dY, CSng(oScreenExtent.Width), CSng(oScreenExtent.Height))
  '  Dim o As PointShape = ExtentHelper.ToWorldCoordinate(PageLayoutMap.MapExtent, dX, dY, CSng(Width), CSng(Height))
  '  Debug.WriteLine("X = " + o.X.ToString + ", Y = " + o.Y.ToString)

  '  Dim oMapPrinterPoint As New PointShape(dX, dY)
  '  Dim oLayerExtent = PageLayoutMap.GetBoundingBox

  '  Dim oPageExtent = PrinterPage.GetBoundingBox
  '  Dim oPageCoord = PrinterHelper.ConvertMapPrinterCoordinateToPageCoordinate(oMapPrinterPoint, PageLayoutMap, CurrentExtent, Width, Height)
  '  'PrinterHelper.
  '  MyBase.OnMapClick(e)
  'End Sub

  Private Sub moPrinterOverlay_MapMouseMove(sender As Object, e As MapMouseMoveInteractiveOverlayEventArgs) Handles moPrinterOverlay.MapMouseMove
    'InMemDisplayLayer.InternalFeatures.Clear()

    Dim oScreenPoint As New ScreenPointF(e.InteractionArguments.ScreenX, e.InteractionArguments.ScreenY)
    Dim oMapPoint As PointShape = PrinterHelper.ConvertScreenCoordinateToMapPrinterCoordinate(
                                                                        oScreenPoint,
                                                                        PageLayoutMap, '<-- MapPrinterLayer
                                                                        e.InteractionArguments.CurrentExtent,
                                                                        e.InteractionArguments.MapWidth,
                                                                        e.InteractionArguments.MapHeight)

    Debug.WriteLine("X = " + oMapPoint.X.ToString + ", Y = " + oMapPoint.Y.ToString)
    Debug.WriteLine("MouseButton = " + e.InteractionArguments.MouseButton.ToString)
    'InMemDisplayLayer.InternalFeatures.Add(oMapPoint.GetFeature)
    'InMemDisplayLayer.BuildIndex()
    'PageLayoutMap.IsDrawing = True

    'PageLayoutMap.IsDrawing = False

    If mbIsLeftMouseButtonDown Then
      '***** Pan the geographical features within the map
      '***** Get the current extent of the map

      '***** Move the extent to where the mouse pointer is
      If moStartPoint IsNot Nothing Then
        Dim dX = moStartPoint.X - oMapPoint.X
        Dim dY = moStartPoint.Y - oMapPoint.Y
        Dim oCurExtent = PageLayoutMap.MapExtent
        oCurExtent = CType(RectangleShape.TranslateByOffset(oCurExtent, dX, dY, GeographyUnit.Meter, DistanceUnit.Meter), RectangleShape)
        PageLayoutMap.MapExtent = oCurExtent
      End If
    End If
      Refresh()
  End Sub
  Private mbIsLeftMouseButtonDown As Boolean = False
  Private moStartPoint As PointShape = Nothing
  Private Sub moPrinterOverlay_MapMouseDown(sender As Object, e As MapMouseDownInteractiveOverlayEventArgs) Handles moPrinterOverlay.MapMouseDown
    mbIsLeftMouseButtonDown = e.InteractionArguments.MouseButton = MapMouseButton.Left
    Dim oScreenPoint As New ScreenPointF(e.InteractionArguments.ScreenX, e.InteractionArguments.ScreenY)
    moStartPoint = PrinterHelper.ConvertScreenCoordinateToMapPrinterCoordinate(
                                                                        oScreenPoint,
                                                                        PageLayoutMap, '<-- MapPrinterLayer
                                                                        e.InteractionArguments.CurrentExtent,
                                                                        e.InteractionArguments.MapWidth,
                                                                        e.InteractionArguments.MapHeight)
  End Sub

  Private Sub moPrinterOverlay_MapMouseUp(sender As Object, e As MapMouseUpInteractiveOverlayEventArgs) Handles moPrinterOverlay.MapMouseUp
    mbIsLeftMouseButtonDown = False
    moStartPoint = Nothing
  End Sub
End Class
